@@ -24,6 +24,7 @@ snd_specific_types = {
     Hinawa.SndUnitType.FIREWORKS:   Hinawa.SndEfw,
     Hinawa.SndUnitType.DIGI00X:     Hinawa.SndDg00x,
     Hinawa.SndUnitType.MOTU:        Hinawa.SndMotu,
+    Hinawa.SndUnitType.TASCAM:      Hinawa.SndTscm,
 }
 for p in Path('/dev/snd/').glob('hw*'):
     fullpath = str(p)
@@ -106,19 +107,18 @@ print(' Config ROM:')
 config_rom = node.get_config_rom()
 quads = unpack('>{}I'.format(len(config_rom) // 4), config_rom)
 for i, q in enumerate(quads):
-    print('  0xfffff000{:04x}: {:08x}'.format(i * 4, q))
+    print('  0xfffff00004{:02x}: {:08x}'.format(i * 4, q))
 
 # create firewire responder
 resp = Hinawa.FwResp()
-def handle_request(resp, tcode):
+def handle_request(resp, tcode, offset, src, dst, card, generation, frame, length):
     print('Requested with tcode: {0}'.format(tcode.value_nick))
-    req_frame = resp.get_req_frame()
-    for i in range(len(req_frame)):
-        print(' [{0:02d}]: 0x{1:02x}'.format(i, req_frame[i]))
+    for i in range(len(frame)):
+        print(' [{0:02d}]: 0x{1:02x}'.format(i, frame[i]))
     return Hinawa.FwRcode.COMPLETE
 try:
     resp.reserve(node, 0xfffff0000d00, 0x100)
-    resp.connect('requested', handle_request)
+    resp.connect('requested2', handle_request)
 except Exception as e:
     print(e)
     exit()
@@ -142,7 +142,7 @@ if unit.get_property('type') in fcp_types:
     request = bytes([0x01, 0xff, 0x19, 0x00, 0xff, 0xff, 0xff, 0xff])
     response = bytearray(8)
     try:
-        response = fcp.transaction(request, response)
+        response = fcp.avc_transaction(request, response, 100)
     except Exception as e:
         print(e)
         exit()
@@ -195,6 +195,12 @@ def handle_motu_notification(self, message):
 if unit.get_property('type') is Hinawa.SndUnitType.MOTU:
     unit.connect('notified', handle_motu_notification)
 
+# Tascam control
+def handle_tscm_control(self, index, before, after):
+    print('control:{}: {:08x}'.format(index, before ^ after))
+if unit.get_property('type') is Hinawa.SndUnitType.TASCAM:
+    unit.connect('control', handle_tscm_control)
+
 # GUI
 class Sample(QWidget):
     def __init__(self, parent=None):
