@@ -14,6 +14,7 @@
  * SECTION:fw_node
  * @Title: HinawaFwNode
  * @Short_description: An event listener for FireWire node
+ * @include: fw_node.h
  *
  * A #HinawaFwNode is an event listener for a specified node on IEEE 1394 bus.
  * This class is an application of Linux FireWire subsystem.
@@ -42,11 +43,30 @@ struct _HinawaFwNodePrivate {
 
 G_DEFINE_TYPE_WITH_PRIVATE(HinawaFwNode, hinawa_fw_node, G_TYPE_OBJECT)
 
-// For error handling.
-G_DEFINE_QUARK("HinawaFwNode", hinawa_fw_node)
-#define raise(exception, errno)						\
-	g_set_error(exception, hinawa_fw_node_quark(), errno,		\
-		    "%d: %s", __LINE__, strerror(errno))
+/**
+ * hinawa_fw_node_error_quark:
+ *
+ * Return the GQuark for error domain of GError which has code in #HinawaFwNodeError.
+ *
+ * Returns: A #GQuark.
+ */
+G_DEFINE_QUARK(hinawa-fw-node-error-quark, hinawa_fw_node_error)
+
+static const char *const err_msgs[] = {
+	[HINAWA_FW_NODE_ERROR_DISCONNECTED] = "The associated node is not available for communication",
+	[HINAWA_FW_NODE_ERROR_OPENED] = "The instance is already associated to node",
+	[HINAWA_FW_NODE_ERROR_NOT_OPENED] = "The instance is not associated to node",
+};
+
+#define generate_local_error(exception, code) \
+	g_set_error_literal(exception, HINAWA_FW_NODE_ERROR, code, err_msgs[code])
+
+#define generate_file_error(exception, code, format, arg) \
+	g_set_error(exception, G_FILE_ERROR, code, format, arg)
+
+#define generate_syscall_error(exception, errno, format, arg)				\
+	g_set_error(exception, HINAWA_FW_NODE_ERROR, HINAWA_FW_NODE_ERROR_FAILED,	\
+		    format " %d(%s)", arg, errno, strerror(errno))
 
 typedef struct {
 	GSource src;
@@ -174,7 +194,7 @@ static void hinawa_fw_node_class_init(HinawaFwNodeClass *klass)
 	 * HinawaFwNode::bus-update:
 	 * @self: A #HinawaFwNode.
 	 *
-	 * When IEEE 1394 bus is updated, the ::bus-update signal is generated.
+	 * When IEEE 1394 bus is updated, the #HinawaFwNode::bus-update signal is generated.
 	 * Handlers can read current generation in the bus via 'generation'
 	 * property.
 	 *
@@ -194,7 +214,7 @@ static void hinawa_fw_node_class_init(HinawaFwNodeClass *klass)
 	 * @self: A #HinawaFwNode.
 	 *
 	 * When phicical FireWire devices are disconnected from IEEE 1394 bus,
-	 * the ::disconnected signal is generated.
+	 * the #HinawaFwNode::disconnected signal is generated.
 	 *
 	 * Since: 1.4.
 	 */
@@ -230,7 +250,7 @@ HinawaFwNode *hinawa_fw_node_new(void)
 	return g_object_new(HINAWA_TYPE_FW_NODE, NULL);
 }
 
-static void update_info(HinawaFwNode *self, GError **exception)
+static int update_info(HinawaFwNode *self)
 {
 	HinawaFwNodePrivate *priv = hinawa_fw_node_get_instance_private(self);
 	struct fw_cdev_get_info info = {0};
@@ -244,10 +264,8 @@ static void update_info(HinawaFwNode *self, GError **exception)
 	info.rom_length = MAX_CONFIG_ROM_LENGTH;
 	info.bus_reset = (__u64)&priv->generation;
 	info.bus_reset_closure = (__u64)self;
-	if (ioctl(priv->fd, FW_CDEV_IOC_GET_INFO, &info) < 0) {
-		raise(exception, errno);
-		return;
-	}
+	if (ioctl(priv->fd, FW_CDEV_IOC_GET_INFO, &info) < 0)
+		return errno;
 
 	// Align buffer for configuration ROM according to host endianness,
 	// because Linux firewire subsystem copies raw data to it.
@@ -256,13 +274,16 @@ static void update_info(HinawaFwNode *self, GError **exception)
 	for (i = 0; i < quads; ++i)
 		rom[i] = GUINT32_FROM_BE(rom[i]);
 	priv->config_rom_length = info.rom_length;
+
+	return 0;
 }
 
 /**
  * hinawa_fw_node_open:
  * @self: A #HinawaFwNode
  * @path: A path to Linux FireWire character device
- * @exception: A #GError
+ * @exception: A #GError. Error can be generated with two domains; #g_file_error_quark(), and
+ *	       #hinawa_fw_node_error_quark().
  *
  * Open Linux FireWire character device to operate node on IEEE 1394 bus.
  *
@@ -272,22 +293,44 @@ void hinawa_fw_node_open(HinawaFwNode *self, const gchar *path,
 			 GError **exception)
 {
 	HinawaFwNodePrivate *priv;
+	int err;
 
 	g_return_if_fail(HINAWA_IS_FW_NODE(self));
-	priv = hinawa_fw_node_get_instance_private(self);
+	g_return_if_fail(path != NULL && strlen(path) > 0);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
+	priv = hinawa_fw_node_get_instance_private(self);
 	if (priv->fd >= 0) {
-		raise(exception, EBUSY);
+		generate_local_error(exception, HINAWA_FW_NODE_ERROR_OPENED);
 		return;
 	}
 
 	priv->fd = open(path, O_RDONLY);
-	if (priv->fd < 0)
-		raise(exception, errno);
+	if (priv->fd < 0) {
+		if (errno == ENODEV) {
+			generate_local_error(exception, HINAWA_FW_NODE_ERROR_DISCONNECTED);
+		} else {
+			GFileError code = g_file_error_from_errno(errno);
+			if (code != G_FILE_ERROR_FAILED)
+				generate_file_error(exception, code, "open(%s)", path);
+			else
+				generate_syscall_error(exception, errno, "open(%s)", path);
+		}
+		return;
+	}
 
 	g_mutex_lock(&priv->mutex);
-	update_info(self, exception);
+	err = update_info(self);
 	g_mutex_unlock(&priv->mutex);
+
+	if (err > 0) {
+		if (err == ENODEV)
+			generate_local_error(exception, HINAWA_FW_NODE_ERROR_DISCONNECTED);
+		else
+			generate_syscall_error(exception, errno, "ioctl(%s)", "FW_CDEV_IOC_GET_INFO");
+		close(priv->fd);
+		priv->fd = -1;
+	}
 }
 
 /**
@@ -308,15 +351,14 @@ void hinawa_fw_node_get_config_rom(HinawaFwNode *self, const guint8 **image,
 	HinawaFwNodePrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_FW_NODE(self));
+	g_return_if_fail(image != NULL);
+	g_return_if_fail(length != NULL);
+	g_return_if_fail(exception == NULL || *exception == NULL);
+
+
 	priv = hinawa_fw_node_get_instance_private(self);
-
 	if (priv->fd < 0) {
-		raise(exception, ENXIO);
-		return;
-	}
-
-	if (image == NULL || length == NULL) {
-		raise(exception, EINVAL);
+		generate_local_error(exception, HINAWA_FW_NODE_ERROR_NOT_OPENED);
 		return;
 	}
 
@@ -328,7 +370,7 @@ void hinawa_fw_node_get_config_rom(HinawaFwNode *self, const guint8 **image,
 	g_mutex_unlock(&priv->mutex);
 }
 
-static void handle_update(HinawaFwNode *self, GError **exception)
+static void handle_update(HinawaFwNode *self)
 {
 	HinawaFwNodePrivate *priv;
 
@@ -336,7 +378,7 @@ static void handle_update(HinawaFwNode *self, GError **exception)
 	priv = hinawa_fw_node_get_instance_private(self);
 
 	g_mutex_lock(&priv->mutex);
-	update_info(self, exception);
+	update_info(self);
 	g_mutex_unlock(&priv->mutex);
 
 	g_signal_emit(self, fw_node_sigs[FW_NODE_SIG_TYPE_BUS_UPDATE], 0, NULL);
@@ -358,8 +400,8 @@ static gboolean dispatch_src(GSource *gsrc, GSourceFunc cb, gpointer user_data)
 	FwNodeSource *src = (FwNodeSource *)gsrc;
 	HinawaFwNodePrivate *priv;
 	GIOCondition condition;
+	struct fw_cdev_event_common *common;
 	ssize_t len;
-	guint8 *buf;
 
 	priv = hinawa_fw_node_get_instance_private(src->self);
 	if (priv->fd < 0)
@@ -380,54 +422,26 @@ static gboolean dispatch_src(GSource *gsrc, GSourceFunc cb, gpointer user_data)
 		return G_SOURCE_REMOVE;
 	}
 
-	buf = src->buf;
-	while (len > 0) {
-		union fw_cdev_event *ev = (union fw_cdev_event *)buf;
-		size_t size = 0;
+	common = (struct fw_cdev_event_common *)src->buf;
 
-		switch (ev->common.type) {
-		case FW_CDEV_EVENT_BUS_RESET:
-			if (HINAWA_IS_FW_NODE(ev->bus_reset.closure)) {
-				GError *exception = NULL;
-				handle_update(src->self, &exception);
-				if (exception != NULL)
-					return G_SOURCE_REMOVE;
-			}
-			size = sizeof(ev->bus_reset);
-			break;
-		case FW_CDEV_EVENT_REQUEST2:
-			if (HINAWA_IS_FW_RESP(ev->request2.closure)) {
-				hinawa_fw_resp_handle_request(
-					HINAWA_FW_RESP(ev->request2.closure),
-					&ev->request2);
-			}
-			size = sizeof(ev->request2) + ev->request2.length;
-			break;
-		case FW_CDEV_EVENT_RESPONSE:
-			if (HINAWA_IS_FW_REQ(ev->response.closure)) {
-				HinawaFwReq *req =
-					HINAWA_FW_REQ(ev->response.closure);
-				GList *entry;
+	if (HINAWA_IS_FW_NODE(common->closure) && common->type == FW_CDEV_EVENT_BUS_RESET) {
+		handle_update(src->self);
+	} else if (HINAWA_IS_FW_RESP(common->closure) && common->type == FW_CDEV_EVENT_REQUEST2) {
+		hinawa_fw_resp_handle_request(HINAWA_FW_RESP(common->closure),
+				(struct fw_cdev_event_request2 *)common);
+	} else if (HINAWA_IS_FW_REQ(common->closure) && common->type == FW_CDEV_EVENT_RESPONSE) {
+		struct fw_cdev_event_response *ev = (struct fw_cdev_event_response *)common;
+		HinawaFwReq *req = HINAWA_FW_REQ(ev->closure);
+		GList *entry;
 
-				// Don't process request invalidated in advance.
-				g_mutex_lock(&priv->transactions_mutex);
-				entry = g_list_find(priv->transactions, req);
-				if (entry) {
-					priv->transactions =
-						g_list_delete_link(priv->transactions,
-								   entry);
-					hinawa_fw_req_handle_response(req, &ev->response);
-				}
-				g_mutex_unlock(&priv->transactions_mutex);
-			}
-			size = sizeof(ev->response) + ev->response.length;
-			break;
-		default:
-			break;
+		// Don't process request invalidated in advance.
+		g_mutex_lock(&priv->transactions_mutex);
+		entry = g_list_find(priv->transactions, req);
+		if (entry) {
+			priv->transactions = g_list_delete_link(priv->transactions, entry);
+			hinawa_fw_req_handle_response(req, ev);
 		}
-
-		len -= size;
-		buf += size;
+		g_mutex_unlock(&priv->transactions_mutex);
 	}
 
 	// Just be sure to continue to process this source.
@@ -445,7 +459,8 @@ static void finalize_src(GSource *gsrc)
  * hinawa_fw_node_create_source:
  * @self: A #HinawaFwNode.
  * @gsrc: (out): A #GSource.
- * @exception: A #GError.
+ * @exception: A #GError. Error can be generated with domain of #hinawa_fw_node_error_quark()
+ *	       and code of #HinawaFwNodeError.
  *
  * Create Gsource for GMainContext to dispatch events for the node on IEEE 1394
  * bus.
@@ -464,10 +479,12 @@ void hinawa_fw_node_create_source(HinawaFwNode *self, GSource **gsrc,
 	FwNodeSource *src;
 
 	g_return_if_fail(HINAWA_IS_FW_NODE(self));
-	priv = hinawa_fw_node_get_instance_private(self);
+	g_return_if_fail(gsrc != NULL);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
+	priv = hinawa_fw_node_get_instance_private(self);
 	if (priv->fd < 0) {
-		raise(exception, ENXIO);
+		generate_local_error(exception, HINAWA_FW_NODE_ERROR_NOT_OPENED);
 		return;
 	}
 
@@ -482,24 +499,24 @@ void hinawa_fw_node_create_source(HinawaFwNode *self, GSource **gsrc,
         // transaction frame.
         src->len = sysconf(_SC_PAGESIZE);
         src->buf = g_malloc0(src->len);
-        if (src->buf == NULL) {
-                raise(exception, ENOMEM);
-		g_source_unref(*gsrc);
-                return;
-        }
 
 	src->self = self;
 	src->tag = g_source_add_unix_fd(*gsrc, priv->fd, G_IO_IN);
 }
 
 // Internal use only.
-void hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args,
-			  int *err)
+int hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args, GError **exception)
 {
 	HinawaFwNodePrivate *priv;
 
-	g_return_if_fail(HINAWA_IS_FW_NODE(self));
+	g_return_val_if_fail(HINAWA_IS_FW_NODE(self), ENXIO);
+	g_return_val_if_fail(exception != NULL, EINVAL);
+
 	priv = hinawa_fw_node_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_FW_NODE_ERROR_NOT_OPENED);
+		return ENXIO;
+	}
 
 	// To invalidate the transaction in a case of timeout.
 	if (req == FW_CDEV_IOC_SEND_REQUEST) {
@@ -508,9 +525,13 @@ void hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args,
 		priv->transactions = g_list_prepend(priv->transactions, req);
 	}
 
-	*err = 0;
-	if (ioctl(priv->fd, req, args) < 0)
-		*err = -errno;
+	if (ioctl(priv->fd, req, args) < 0) {
+		if (errno == ENODEV)
+			generate_local_error(exception, HINAWA_FW_NODE_ERROR_DISCONNECTED);
+		return errno;
+	}
+
+	return 0;
 }
 
 void hinawa_fw_node_invalidate_transaction(HinawaFwNode *self, HinawaFwReq *req)
