@@ -1,17 +1,13 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
-#ifndef __ALSA_TOOLS_HINAWA_INTERNAL_H__
-#define __ALSA_TOOLS_HINAWA_INTERNAL_H__
+#ifndef __ALSA_HINAWA_INTERNAL_H__
+#define __ALSA_HINAWA_INTERNAL_H__
 
 #include <config.h>
 
 #include <linux/firewire-cdev.h>
 #include <linux/firewire-constants.h>
 
-#if WITH_BACKPORT
-#include "backport.h"
-#else
 #include <sound/firewire.h>
-#endif
 
 #include "fw_node.h"
 #include "fw_resp.h"
@@ -23,8 +19,7 @@
 #include "snd_motu.h"
 #include "snd_tscm.h"
 
-void hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args,
-			  int *err);
+int hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args, GError **exception);
 void hinawa_fw_node_invalidate_transaction(HinawaFwNode *self, HinawaFwReq *req);
 
 void hinawa_fw_resp_handle_request(HinawaFwResp *self,
