@@ -1,10 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
-#ifndef __ALSA_TOOLS_HINAWA_SND_DICE_H__
-#define __ALSA_TOOLS_HINAWA_SND_DICE_H__
+#ifndef __ALSA_HINAWA_SND_DICE_H__
+#define __ALSA_HINAWA_SND_DICE_H__
 
 #include <glib.h>
 #include <glib-object.h>
-#include "snd_unit.h"
+
+#include <snd_unit.h>
 
 G_BEGIN_DECLS
 
@@ -30,6 +31,10 @@ G_BEGIN_DECLS
 				   HINAWA_TYPE_SND_DICE,	\
 				   HinawaSndDiceClass))
 
+#define HINAWA_SND_DICE_ERROR	hinawa_snd_dice_error_quark()
+
+GQuark hinawa_snd_dice_error_quark();
+
 typedef struct _HinawaSndDice		HinawaSndDice;
 typedef struct _HinawaSndDiceClass	HinawaSndDiceClass;
 typedef struct _HinawaSndDicePrivate	HinawaSndDicePrivate;
@@ -48,7 +53,7 @@ struct _HinawaSndDiceClass {
 	 * @self: A #HinawaSndDice
 	 * @message: A notification message
 	 *
-	 * When Dice models transfer notification, the ::notified handler is
+	 * When Dice models transfer notification, the #HinawaSndDiceClass::notified handler is
 	 * called.
 	 */
 	void (*notified)(HinawaSndDice *self, guint message);
