@@ -1,10 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
-#ifndef __ALSA_TOOLS_HINAWA_SND_MOTU_H__
-#define __ALSA_TOOLS_HINAWA_SND_MOTU_H__
+#ifndef __ALSA_HINAWA_SND_MOTU_H__
+#define __ALSA_HINAWA_SND_MOTU_H__
 
 #include <glib.h>
 #include <glib-object.h>
-#include "snd_unit.h"
+
+#include <snd_unit.h>
 
 G_BEGIN_DECLS
 
@@ -48,7 +49,7 @@ struct _HinawaSndMotuClass {
 	 * @self: A #HinawaSndMotu
 	 * @message: A notification message
 	 *
-	 * When Motu models transfer notification, the ::notified handler is
+	 * When Motu models transfer notification, the #HinawaSndMotuClass::notified handler is
 	 * called.
 	 */
 	void (*notified)(HinawaSndMotu *self, guint message);
