@@ -1,10 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
-#ifndef __ALSA_TOOLS_HINAWA_SND_TSCM_H__
-#define __ALSA_TOOLS_HINAWA_SND_TSCM_H__
+#ifndef __ALSA_HINAWA_SND_TSCM_H__
+#define __ALSA_HINAWA_SND_TSCM_H__
 
 #include <glib.h>
 #include <glib-object.h>
-#include "snd_unit.h"
+
+#include <snd_unit.h>
 
 G_BEGIN_DECLS
 
@@ -50,8 +51,8 @@ struct _HinawaSndTscmClass {
 	 * @before: the value of info before changed.
 	 * @after: the value of info after changed.
 	 *
-	 * When TASCAM FireWire unit transfer control message, the ::control
-	 * signal is emitted.
+	 * When TASCAM FireWire unit transfer control message, the #HinawaSndTscmClass::control
+	 * handler is emitted.
 	 */
 	void (*control)(HinawaSndTscm *self, guint index, guint before,
 			guint after);
