@@ -17,16 +17,40 @@
  * SECTION:snd_unit
  * @Title: HinawaSndUnit
  * @Short_description: An event listener for ALSA FireWire sound devices
+ * @include: snd_unit.h
  *
  * This class is an application of ALSA FireWire stack. Any functionality which
  * ALSA drivers in the stack can be available.
  */
 
-/* For error handling. */
-G_DEFINE_QUARK("HinawaSndUnit", hinawa_snd_unit)
-#define raise(exception, errno)						\
-	g_set_error(exception, hinawa_snd_unit_quark(), errno,		\
-		    "%d: %s", __LINE__, strerror(errno))
+/**
+ * hinawa_snd_unit_error_quark:
+ *
+ * Return the GQuark for error domain of GError which has code in #HinawaSndUnitError.
+ *
+ * Returns: A #GQuark.
+ */
+G_DEFINE_QUARK(hinawa-snd-unit-error-quark, hinawa_snd_unit_error)
+
+static const char *const err_msgs[] = {
+	[HINAWA_SND_UNIT_ERROR_DISCONNECTED] = "The associated hwdep device is not available",
+	[HINAWA_SND_UNIT_ERROR_USED] = "The hedep device is already in use",
+	[HINAWA_SND_UNIT_ERROR_OPENED] = "The instance is already associated to unit",
+	[HINAWA_SND_UNIT_ERROR_NOT_OPENED] = "The instance is not associated to unit yet",
+	[HINAWA_SND_UNIT_ERROR_LOCKED] = "The associated hwdep device is already locked or kernel packet streaming runs",
+	[HINAWA_SND_UNIT_ERROR_UNLOCKED] = "The associated hwdep device is not locked against kernel packet streaming",
+	[HINAWA_SND_UNIT_ERROR_WRONG_CLASS] = "The hwdep device is not for the unit expected by the class",
+};
+
+#define generate_local_error(exception, code) \
+	g_set_error_literal(exception, HINAWA_FW_NODE_ERROR, code, err_msgs[code])
+
+#define generate_file_error(exception, code, format, arg) \
+	g_set_error(exception, G_FILE_ERROR, code, format, arg)
+
+#define generate_syscall_error(exception, errno, format, arg)				\
+	g_set_error(exception, HINAWA_SND_UNIT_ERROR, HINAWA_SND_UNIT_ERROR_FAILED,	\
+		    format " %d(%s)", arg, errno, strerror(errno))
 
 struct _HinawaSndUnitPrivate {
 	int fd;
@@ -99,6 +123,7 @@ static void snd_unit_finalize(GObject *obj)
 	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(self);
 
 	close(priv->fd);
+	priv->fd = -1;
 	g_object_unref(priv->node);
 
 	G_OBJECT_CLASS(hinawa_snd_unit_parent_class)->finalize(obj);
@@ -148,7 +173,7 @@ static void hinawa_snd_unit_class_init(HinawaSndUnitClass *klass)
 	 * @self: A #HinawaSndUnit
 	 * @state: %TRUE when locked, %FALSE when unlocked.
 	 *
-	 * When ALSA kernel-streaming status is changed, this ::lock-status
+	 * When ALSA kernel-streaming status is changed, this #HinawaSndUnit::lock-status
 	 * signal is generated.
 	 */
 	snd_unit_sigs[SND_UNIT_SIG_TYPE_LOCK_STATUS] =
@@ -183,6 +208,7 @@ static void hinawa_snd_unit_init(HinawaSndUnit *self)
 {
 	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(self);
 
+	priv->fd = -1;
 	priv->node = g_object_new(HINAWA_TYPE_FW_NODE, NULL);
 }
 
@@ -203,7 +229,8 @@ HinawaSndUnit *hinawa_snd_unit_new(void)
  * hinawa_snd_unit_open:
  * @self: A #HinawaSndUnit
  * @path: A full path of a special file for ALSA hwdep character device
- * @exception: A #GError
+ * @exception: A #GError. Error can be generated with three domains; #g_file_error_quark(),
+ *	       #hinawa_fw_node_error_quark(), and #hinawa_snd_unit_error_quark().
  *
  * Open ALSA hwdep character device and check it for FireWire sound devices.
  */
@@ -213,25 +240,78 @@ void hinawa_snd_unit_open(HinawaSndUnit *self, gchar *path, GError **exception)
 	char fw_cdev[32];
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	g_return_if_fail(path != NULL && strlen(path) > 0);
+	g_return_if_fail(exception == NULL || *exception == NULL);
+
 	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd >= 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_OPENED);
+		return;
+	}
 
 	priv->fd = open(path, O_RDWR);
 	if (priv->fd < 0) {
-		raise(exception, errno);
-		goto end;
+		if (errno == ENODEV) {
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		} else if (errno == EBUSY) {
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_USED);
+		} else {
+			GFileError code = g_file_error_from_errno(errno);
+
+			if (code != G_FILE_ERROR_FAILED)
+				generate_file_error(exception, code, "open(%s)", path);
+			else
+				generate_syscall_error(exception, errno, "open(%s)", path);
+		}
+		return;
 	}
 
 	/* Get FireWire sound device information. */
 	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_GET_INFO, &priv->info) < 0) {
-		raise(exception, errno);
+		if (errno == ENODEV)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		else
+			generate_syscall_error(exception, errno, "ioctl(%s)", "SNDRV_FIREWIRE_IOCTL_GET_INFO");
 		goto end;
 	}
 
+#if HAVE_SND_DICE
+	if (HINAWA_IS_SND_DICE(self) && priv->info.type != SNDRV_FIREWIRE_TYPE_DICE) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_WRONG_CLASS);
+		goto end;
+	}
+#endif
+#if HAVE_SND_EFW
+	if (HINAWA_IS_SND_EFW(self) && priv->info.type != SNDRV_FIREWIRE_TYPE_FIREWORKS) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_WRONG_CLASS);
+		goto end;
+	}
+#endif
+#if HAVE_SND_DG00X
+	if (HINAWA_IS_SND_DG00X(self) && priv->info.type != SNDRV_FIREWIRE_TYPE_DIGI00X) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_WRONG_CLASS);
+		goto end;
+	}
+#endif
+#if HAVE_SND_MOTU
+	if (HINAWA_IS_SND_MOTU(self) && priv->info.type != SNDRV_FIREWIRE_TYPE_MOTU) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_WRONG_CLASS);
+		goto end;
+	}
+#endif
+#if HAVE_SND_TSCM
+	if (HINAWA_IS_SND_TSCM(self) && priv->info.type != SNDRV_FIREWIRE_TYPE_TASCAM) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_WRONG_CLASS);
+		goto end;
+	}
+#endif
 	snprintf(fw_cdev, sizeof(fw_cdev), "/dev/%s", priv->info.device_name);
 	hinawa_fw_node_open(priv->node, fw_cdev, exception);
 end:
-	if (*exception != NULL)
+	if (*exception != NULL) {
 		close(priv->fd);
+		priv->fd = -1;
+	}
 }
 
 /**
@@ -248,14 +328,17 @@ void hinawa_snd_unit_get_node(HinawaSndUnit *self, HinawaFwNode **node)
 	HinawaSndUnitPrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	g_return_if_fail(node != NULL);
+
 	priv = hinawa_snd_unit_get_instance_private(self);
+	g_return_if_fail(priv->fd >= 0);
 
 	*node = priv->node;
 }
 /**
  * hinawa_snd_unit_lock:
  * @self: A #HinawaSndUnit
- * @exception: A #GError
+ * @exception: A #GError. Error can be generated with domain of #hinawa_snd_unit_error_quark().
  *
  * Disallow ALSA to start kernel-streaming.
  */
@@ -264,16 +347,28 @@ void hinawa_snd_unit_lock(HinawaSndUnit *self, GError **exception)
 	HinawaSndUnitPrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
-	priv = hinawa_snd_unit_get_instance_private(self);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
-	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_LOCK, NULL) < 0)
-		raise(exception, errno);
+	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_NOT_OPENED);
+		return;
+	}
+
+	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_LOCK, NULL) < 0) {
+		if (errno == ENODEV)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		else if (errno == EBUSY)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_LOCKED);
+		else
+			generate_syscall_error(exception, errno, "ioctl(%s)", "SNDRV_FIREWIRE_IOCTL_LOCK");
+	}
 }
 
 /**
  * hinawa_snd_unit_unlock:
  * @self: A #HinawaSndUnit
- * @exception: A #GError
+ * @exception: A #GError. Error can be generated with domain of #hinawa_snd_unit_error_quark().
  *
  * Allow ALSA to start kernel-streaming.
  */
@@ -282,10 +377,22 @@ void hinawa_snd_unit_unlock(HinawaSndUnit *self, GError **exception)
 	HinawaSndUnitPrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
-	priv = hinawa_snd_unit_get_instance_private(self);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
-	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL) < 0)
-		raise(exception, errno);
+	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_NOT_OPENED);
+		return;
+	}
+
+	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL) < 0) {
+		if (errno == ENODEV)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		else if (errno == EBADFD)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_UNLOCKED);
+		else
+			generate_syscall_error(exception, errno, "ioctl(%s)", "SNDRV_FIREWIRE_IOCTL_UNLOCK");
+	}
 }
 
 /* For internal use. */
@@ -293,12 +400,26 @@ void hinawa_snd_unit_write(HinawaSndUnit *self, const void *buf, size_t length,
 			   GError **exception)
 {
 	HinawaSndUnitPrivate *priv;
+	ssize_t len;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
-	priv = hinawa_snd_unit_get_instance_private(self);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
-	if (write(priv->fd, buf, length) != length)
-		raise(exception, errno);
+	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_NOT_OPENED);
+		return;
+	}
+
+	len = write(priv->fd, buf, length);
+	if (len < 0) {
+		if (errno == ENODEV)
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		else
+			generate_syscall_error(exception, errno, "write: %zu", length);
+	} else if (len != length){
+		generate_syscall_error(exception, EIO, "write: %zu", length);
+	}
 }
 
 void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
@@ -307,10 +428,34 @@ void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
 	HinawaSndUnitPrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
-	priv = hinawa_snd_unit_get_instance_private(self);
+	g_return_if_fail(exception == NULL || *exception == NULL);
 
-	if (ioctl(priv->fd, request, arg) < 0)
-		raise(exception, errno);
+	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_NOT_OPENED);
+		return;
+	}
+
+	if (ioctl(priv->fd, request, arg) < 0) {
+		if (errno == ENODEV) {
+			generate_local_error(exception, HINAWA_SND_UNIT_ERROR_DISCONNECTED);
+		} else {
+			const char *arg;
+
+			switch (request) {
+#if HAVE_SND_TSCM
+			case SNDRV_FIREWIRE_IOCTL_TASCAM_STATE:
+				arg = "SNDRV_FIREWIRE_IOCTL_TASCAM_STATE";
+				break;
+#endif
+			default:
+				arg = "Unknown";
+				break;
+			}
+
+			generate_syscall_error(exception, errno, "ioctl(%s)", arg);
+		}
+	}
 }
 
 static void handle_lock_event(HinawaSndUnit *self,
@@ -417,7 +562,7 @@ static void finalize_src(GSource *gsrc)
  * hinawa_snd_unit_create_source:
  * @self: A #HinawaSndUnit.
  * @gsrc: (out): A #GSource.
- * @exception: A #GError.
+ * @exception: A #GError. Error can be generated with domain with #hinawa_snd_unit_error_quark().
  *
  * Create Gsource for GMainContext to dispatch events for the sound device.
  *
@@ -435,7 +580,14 @@ void hinawa_snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
 	SndUnitSource *src;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	g_return_if_fail(gsrc != NULL);
+	g_return_if_fail(exception == NULL || *exception == NULL);
+
 	priv = hinawa_snd_unit_get_instance_private(self);
+	if (priv->fd < 0) {
+		generate_local_error(exception, HINAWA_SND_UNIT_ERROR_NOT_OPENED);
+		return;
+	}
 
 	*gsrc = g_source_new(&funcs, sizeof(SndUnitSource));
 
@@ -448,11 +600,6 @@ void hinawa_snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
 	src = (SndUnitSource *)(*gsrc);
 	src->len = sysconf(_SC_PAGESIZE);
 	src->buf = g_malloc(src->len);
-	if (src->buf == NULL) {
-		raise(exception, ENOMEM);
-		g_source_unref(*gsrc);
-		return;
-	}
 
 	src->unit = self;
 	src->tag = g_source_add_unix_fd(*gsrc, priv->fd, G_IO_IN);
