@@ -1,10 +1,12 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
-#ifndef __ALSA_TOOLS_HINAWA_SND_UNIT_H__
-#define __ALSA_TOOLS_HINAWA_SND_UNIT_H__
+#ifndef __ALSA_HINAWA_SND_UNIT_H__
+#define __ALSA_HINAWA_SND_UNIT_H__
 
 #include <glib.h>
 #include <glib-object.h>
 
+#include <fw_node.h>
+
 G_BEGIN_DECLS
 
 #define HINAWA_TYPE_SND_UNIT	(hinawa_snd_unit_get_type())
@@ -29,6 +31,10 @@ G_BEGIN_DECLS
 				   HINAWA_TYPE_SND_UNIT,	\
 				   HinawaSndUnitClass))
 
+#define HINAWA_SND_UNIT_ERROR	hinawa_snd_unit_error_quark()
+
+GQuark hinawa_snd_unit_error_quark();
+
 typedef struct _HinawaSndUnit		HinawaSndUnit;
 typedef struct _HinawaSndUnitClass	HinawaSndUnitClass;
 typedef struct _HinawaSndUnitPrivate	HinawaSndUnitPrivate;
@@ -47,7 +53,7 @@ struct _HinawaSndUnitClass {
 	 * @self: A #HinawaSndUnit
 	 * @state: %TRUE when locked, %FALSE when unlocked.
 	 *
-	 * When ALSA kernel-streaming status is changed, this ::lock_status
+	 * When ALSA kernel-streaming status is changed, this #HinawaSndUnitClass::lock_status
 	 * handler is called.
 	 */
 	void (*lock_status)(HinawaSndUnit *self, gboolean state);
